package com.simuwang.manage.api.template;

import com.simuwang.base.common.support.MybatisPage;
import com.simuwang.base.pojo.dto.query.DeletionPageQuery;
import com.simuwang.base.pojo.dto.query.EmailTemplateInfoPageQuery;
import com.simuwang.base.pojo.vo.*;
import com.simuwang.logging.SystemLog;
import com.simuwang.manage.service.EmailTemplateInfoService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

/**
 * FileName: TemplateController
 * Author:   chenjianhua
 * Date:     2024/9/25 18:46
 * Description: ${DESCRIPTION}
 */
@RestController
@RequestMapping("/v1/template")
public class TemplateController {

    @Autowired
    private EmailTemplateInfoService emailTemplateInfoService;

    /**
     * 模版详情页面展示数据
     * @param emailTemplateInfoPageQuery
     * @return
     */
    @SystemLog(value = "模版详情页面展示数据")
    @RequestMapping("search-template-list")
    public MybatisPage<EmailTemplateInfoVO> searchTemplateList(EmailTemplateInfoPageQuery emailTemplateInfoPageQuery){
        MybatisPage<EmailTemplateInfoVO> result = emailTemplateInfoService.searchTemplateList(emailTemplateInfoPageQuery);
        return result;
    }


    /**
     * 批量删除模版
     * @param idListVO
     * @return
     */
    @SystemLog(value = "批量删除模版")
    @RequestMapping("delete-template-list")
    public boolean deleteTemplateList(@RequestBody IdListVO idListVO){
        emailTemplateInfoService.deleteTemplateList(idListVO);
        return true;
    }


    /**
     * 批量删除模版
     * @param saveTemplateInfoVO
     * @return
     */
    @SystemLog(value = "保存模版信息")
    @RequestMapping("save-template-list")
    public boolean saveTemplateList(@RequestBody SaveTemplateInfoVO saveTemplateInfoVO){
        emailTemplateInfoService.saveTemplateList(saveTemplateInfoVO);
        return true;
    }

    /**
     * 批量删除适用规则
     * @param idListVO
     * @return
     */
    @SystemLog(value = "批量删除适用规则")
    @RequestMapping("delete-application-rule")
    public boolean deleteApplicationRuleList(@RequestBody IdListVO idListVO){
        emailTemplateInfoService.deleteApplicationRuleList(idListVO);
        return true;
    }

}
